<?php

declare(strict_types=1);

namespace Drupal\state_machine_automated_transition\Hook;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\state_machine\Plugin\Workflow\WorkflowTransition;
use Drupal\state_machine_automated_transition\AutomatedTransitionOrchestrator;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Hook implementations for the node module.
 */
class EntityUpdate {

  /**
   * The immutable config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  public function __construct(
    #[Autowire(service: 'state_machine_automated_transition.orchestrator')]
    private readonly AutomatedTransitionOrchestrator $automatedTransitionOrchestrator,
    #[Autowire(service: 'entity_field.manager')]
    private readonly EntityFieldManager $entityFieldManager,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->config = $configFactory->get('state_machine_automated_transition.settings');
  }

  /**
   * Implements hook_entity_update().
   *
   * Applies consecutive transition at the end of the current transition.
   */
  #[Hook('entity_update')]
  public function entityUpdate(EntityInterface $entity) {
    $allow_list = $this->config->get('allow_list');
    // Find field.
    $fields = $this->entityFieldManager->getFieldMapByFieldType('state');
    $fields = $fields[$entity->getEntityTypeId()] ?? [];
    foreach ($fields as $field_name => $field) {
      if (!$entity->hasField($field_name)) {
        continue;
      }
      // @todo: If value didn't change, skip.
      if (FALSE) {
        continue;
      }
      /** @var \Drupal\state_machine\Plugin\Field\FieldType\StateItemInterface $state */
      $state = $entity->get($field_name)->first();
      $transitions = $state->getTransitions();
      $workflow = $state->getWorkflow();
      // Use array_filter to only keep transitions that are allowed.
      $transitions = array_filter($transitions, function (WorkflowTransition $transition) use ($allow_list, $workflow) {
        return !empty($allow_list[$workflow->getGroup()][$workflow->getId()][$transition->getId()]);
      });
      // Reset array to keep only first item.
      // @todo: Dispatch warning log entry.
      $transition = reset($transitions);
      if (!$transition instanceof WorkflowTransition) {
        continue;
      }
      $this->automatedTransitionOrchestrator->addTransition($entity, $field_name, $transition);
    }
  }
}
